package app

import (
	"context"
	"os"
	"testing"

	. "my/app/pkg/testlib"

	"github.com/cucumber/godog"
	"github.com/cucumber/godog/colors"

	. "github.com/onsi/gomega"
)

func InitializeScenario(ctx *godog.ScenarioContext) {

	// -----------------------
	ctx.Before(func(ctx context.Context, sc *godog.Scenario) (context.Context, error) {
		beforeScenario()
		return ctx, nil
	})
	ctx.After(func(ctx context.Context, sc *godog.Scenario, err error) (context.Context, error) {
		afterScenario()
		return ctx, nil
	})
	InitializeGomegaForGodog(ctx)
	_ = Ω
}

func InitializeSuite(tsc *godog.TestSuiteContext) {
	tsc.BeforeSuite(beforeSuite)
	tsc.AfterSuite(afterSuite)
}

func TestMain(m *testing.M) {
	var opts = godog.Options{
		Output:        colors.Colored(os.Stdout),
		Strict:        true,
		StopOnFailure: true,
	}

	godog.BindCommandLineFlags("godog.", &opts)
	status := godog.TestSuite{
		Name:                 "app",
		TestSuiteInitializer: InitializeSuite,
		ScenarioInitializer:  InitializeScenario,
		Options:              &opts,
	}.Run()
}
